@@ -15,7 +15,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2015 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -58,6 +58,7 @@ npr.seed(3)
 
 IS_PLOTTING = True
 fsize = (6,4.5)
+STORE_DATA = False
 IS_STORING = False
 DAT_FOLDER = "./Data-qttdmrg"
 FIG_FOLDER = "./Fig-tmp-qttdmrg"
@@ -95,39 +96,43 @@ def test(FNUM, GenzNormalized=False, SpectralType='Projection', GenzPatrick=Fals
 
         if FNUM == 0:
             FUNC = 0
-            ds = [10,50,100]
-            stt_types = [SQTT,SQTT,SQTT]
-            kickrank = [None]*3
+            # ds = [10,50,100]
+            # stt_types = [SQTT,SQTT,SQTT]
+            ds = [10]
+            stt_types = [SQTT]
+            kickrank = None
             eps = 1e-10
         elif FNUM == 1:
             FUNC = 1
             ds = [10,15,20]
             stt_types = [SQTT,SQTT,SQTT]
-            kickrank = [None]*3
+            kickrank = None
             eps = 1e-10
         elif FNUM == 2:
             FUNC = 2
             ds = [5,10,15]
             stt_types = [SQTT,SQTT,SQTT]
-            kickrank = [None]*3
+            # ds = [10]
+            # stt_types = [SQTT]
+            kickrank = None
             eps = 1e-10
         elif FNUM == 3:
             FUNC = 3
             ds = [10,50,100]
             stt_types = [SQTT,SQTT,SQTT]
-            kickrank = [None]*3
+            kickrank = None
             eps = 1e-10
         elif FNUM == 4:
             FUNC = 4
             ds = [10,50,100]
             stt_types = [SQTT,SQTT,SQTT]
-            kickrank = [None]*3
+            kickrank = None
             eps = 1e-10
         elif FNUM == 5:
             FUNC = 5
             ds = [10,15,20]
             stt_types = [SQTT,SQTT,SQTT]
-            kickrank = [None]*3
+            kickrank = None
             eps = 1e-10
 
     if GenzPatrick:
@@ -150,25 +155,25 @@ def test(FNUM, GenzNormalized=False, SpectralType='Projection', GenzPatrick=Fals
             FUNC = 0
             ds = [5]
             stt_types = [SQTT]
-            kickrank = [None]
+            kickrank = None
             eps = 1e-10
         elif FNUM == 1:
             FUNC = 1
             ds = [5]
             stt_types = [SQTT]
-            kickrank = [None]
+            kickrank = None
             eps = 1e-10
         elif FNUM == 2:
             FUNC = 2
             ds = [5]
             stt_types = [SQTT]
-            kickrank = [None]
+            kickrank = None
             eps = 1e-10
         elif FNUM == 3:
             FUNC = 3
             ds = [5]
             stt_types = [SQTT]
-            kickrank = [None]
+            kickrank = None
             eps = 1e-10
 
     print "Function: " + str(FUNC) + " Norm: " + str(GenzNormalized) + " Dims: " + str(ds) + " Type: " + SpectralType
@@ -339,12 +344,12 @@ def test(FNUM, GenzNormalized=False, SpectralType='Projection', GenzPatrick=Fals
                     if stt_type == STT:
                         STTapprox = DT.STT(f, X, params, orders=size,
                                            method='ttdmrg', eps=eps, mv_eps=maxvoleps,maxit=tt_maxit,
-                                           kickrank=2,
+                                           kickrank=None,
                                            surrogateONOFF=True, surrogate_type=surr_type)
                     elif stt_type == SQTT:
                         STTapprox = DT.SQTT(f, X, params, orders=size,
                                             method='ttdmrg', eps=eps, mv_eps=maxvoleps,maxit=tt_maxit,
-                                            kickrank=2,
+                                            kickrank=None,
                                             surrogateONOFF=True, surrogate_type=surr_type)
 
                     STTapprox.build()
@@ -391,7 +396,7 @@ def test(FNUM, GenzNormalized=False, SpectralType='Projection', GenzPatrick=Fals
                         # mean = vol * np.mean(values)
                         # var = vol**2. * np.var(values) / len(values)
 
-                        sys.stdout.write("L2err estim. iter: %d Var: %e VarLim: %e L2err: %e \r" % (len(values) , VarI,  MCestVarLimit, np.sqrt(EstI)/np.sqrt(np.mean(intf)) ))
+                        sys.stdout.write("L2err estim. iter: %d Var: %e VarLim: %e L2err: %e Feval: %d \r" % (len(values) , VarI,  MCestVarLimit, np.sqrt(EstI)/np.sqrt(np.mean(intf)), feval[i_d,n_exp,i_size] ))
                         sys.stdout.flush()
 
                     sys.stdout.write("\n")
@@ -411,10 +416,11 @@ def test(FNUM, GenzNormalized=False, SpectralType='Projection', GenzPatrick=Fals
                         path = DAT_FOLDER + "/" + SpectralType + "-" + file_names[FUNC] + file_name_ext + "-" + "data-Patrick.pkl"
                     else:
                         path = DAT_FOLDER + "/" + SpectralType + "-" + file_names[FUNC] + file_name_ext + "-" + "data.pkl"
-
-                    outfile = open(path,'wb')
-                    pkl.dump(data,outfile)
-                    outfile.close()
+                    
+                    if STORE_DATA:
+                        outfile = open(path,'wb')
+                        pkl.dump(data,outfile)
+                        outfile.close()
 
 
     if IS_PLOTTING:
