@@ -19,7 +19,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -28,15 +28,15 @@
 
 __all__ = []
 
-from _version import __version__
+from TensorToolbox._version import __version__
 
-import core
-from core import *
+import TensorToolbox.core
+from TensorToolbox.core import *
 
-import multilinalg
+from TensorToolbox import multilinalg
 
-import unittests
-from unittests import *
+import TensorToolbox.unittests
+from TensorToolbox.unittests import *
 
 __all__.extend(['multilinalg'])
 
