@@ -15,13 +15,14 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
 # Author: Daniele Bigoni
 #
 
+
 __all__ = ['QTTmat']
 
 import logging
@@ -29,10 +30,10 @@ import operator
 import numpy as np
 from TensorToolbox.core import QTTvec
 from TensorToolbox.core import TTmat
-from TensorToolbox.core import idxfold, idxunfold
+from TensorToolbox.core import idxfold, idxunfold, isint
 
 class QTTmat(TTmat):
-    """ Constructor of multidimensional matrix in Quantics Tensor Train format
+    """ Constructor of multidimensional matrix in Quantics Tensor Train format :cite:`Khoromskij2010,Khoromskij2011`.
 
     :param ndarray,TT A: Available input formats are full tensor in numpy.ndarray, Tensor Train structure (list of cores). If input is ndarray, then it must be in mattensor format (see aux.py)
     :param int base: folding base for QTT representation
@@ -75,11 +76,11 @@ class QTTmat(TTmat):
     def __setstate__(self,state):
         super(QTTmat,self).__setstate__( state )
         
-    def build(self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, kickrank=2):
+    def build(self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, max_ranks=None, kickrank=2):
         """ Common interface for the construction of the approximation.
 
         :param float eps: [default == 1e-10] For method=='svd': precision with which to approximate the input tensor. For method=='ttcross': TT-rounding tolerance for rank-check.
-        :param string method: 'svd' use singular value decomposition to construct the TT representation [1]_, 'ttcross' use low rank skeleton approximation to construct the TT representation [2]_, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation [3]_ [4]_, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
+        :param string method: 'svd' use singular value decomposition to construct the TT representation :cite:`Oseledets2011`, 'ttcross' use low rank skeleton approximation to construct the TT representation :cite:`Oseledets2010`, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation :cite:`Savostyanov2011,Savostyanov2013`, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
         :param list rs: list of integer ranks of different cores. If ``None`` then the incremental TTcross approach will be used. (method=='ttcross')
         :param bool fix_rank: determines whether the rank is allowed to be increased (method=='ttcross')
         :param list Jinit: list of list of integers containing the r starting columns in the lowrankapprox routine for each core. If ``None`` then pick them randomly. (method=='ttcross')
@@ -88,13 +89,14 @@ class QTTmat(TTmat):
         :param float mv_eps: accuracy parameter for each usage of the maxvol algorithm (method=='ttcross')
         :param int mv_maxit: maximum number of iterations in the maxvol routine (method=='ttcross')
         :param bool fix_rank: Whether the rank is allowed to increase
+        :param list max_ranks: Maximum ranks to be used to limit the trunaction rank due to ``eps``. The first and last elements of the list must be ``1``, e.g. ``[1,...,1]``. Default: ``None``.
         :param int kickrank: rank overshooting for 'ttdmrg'
         """
         
         nrows = self.full_nrows
         ncols = self.full_ncols
         
-        if isinstance(nrows,int) and isinstance(ncols,int):
+        if isint(nrows) and isint(ncols):
             nrows = [nrows]
             ncols = [ncols]
 
@@ -127,11 +129,11 @@ class QTTmat(TTmat):
             self.A = np.transpose(self.A,axes=idxs)
             self.A = np.reshape(self.A,[self.basemat for i in range(self.L)])
             
-            super(QTTmat,self).build( eps, method, rs, fix_rank, Jinit, delta, maxit, mv_eps, mv_maxit, kickrank )
+            super(QTTmat,self).build( eps=eps, method=method, rs=rs, fix_rank=fix_rank, Jinit=Jinit, delta=delta, maxit=maxit, mv_eps=mv_eps, mv_maxit=mv_maxit, max_ranks=max_ranks, kickrank=kickrank )
             
         elif isinstance(self.A,list):
             
-            super(QTTmat,self).build( eps, method, rs, fix_rank, Jinit, delta, maxit, mv_eps, mv_maxit, kickrank )
+            super(QTTmat,self).build( eps=eps, method=method, rs=rs, fix_rank=fix_rank, Jinit=Jinit, delta=delta, maxit=maxit, mv_eps=mv_eps, mv_maxit=mv_maxit, max_ranks=max_ranks, kickrank=kickrank )
             
             # Check that unfolded nrows,ncols are consistent with the dimension of A
             if np.prod(self.shape()) != np.prod(self.full_nrows)*np.prod(self.full_ncols):
