@@ -15,7 +15,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -29,14 +29,16 @@ import warnings
 import logging 
 import operator
 import numpy as np
+import numpy.linalg as npla
 import math
 import itertools
 
-from TensorToolbox.core import TTvec, idxfold, idxunfold, TensorWrapper, matkron_to_mattensor
+from TensorToolbox.core import TTvec, idxfold, idxunfold, expand_idxs, \
+    TensorWrapper, matkron_to_mattensor, isint
 from TensorToolbox import multilinalg as mla
 
 class QTTvec(TTvec):
-    """ Constructor of multidimensional tensor in Quantics Tensor Train format
+    """ Constructor of multidimensional tensor in Quantics Tensor Train format :cite:`Khoromskij2010,Khoromskij2011`.
         
     :param ndarray,TensorWrapper,TT A: Available input formats are full tensor in numpy.ndarray, Tensor Wrapper, Tensor Train structure (list of cores)
     :param list global_shape: Argument to be provided if ``A`` is the list of cores of a TT format.
@@ -108,15 +110,16 @@ class QTTvec(TTvec):
                         idxs_out = tuple(idxs_out)
                         idxs_in = tuple(idxs_in)
                         Anew[ idxs_out ] = self.A[ idxs_in ]
+                        # Anew[ idxs_out ] = TensorWrapper.FILL_VALUE
                 self.A = Anew
             else:
                 self.A.set_Q( base )
                 
             # Set the folded_shape (list of list) for each dimension
-            self.folded_shape = [ [self.base] * int(math.log(self.A.shape[i],self.base)) for i in xrange(len(self.global_shape)) ]
+            self.folded_shape = [ [self.base] * int(round(math.log(self.A.shape[i],self.base))) for i in range(len(self.global_shape)) ]
             
             # Folding matrix
-            new_shape = [self.base] * int(math.log( self.A.size, self.base ))
+            new_shape = [self.base] * int(round(math.log( self.A.size, self.base )))
             self.A = self.A.reshape(new_shape)
             
             # # Construct the folding shape
@@ -148,11 +151,11 @@ class QTTvec(TTvec):
     def __setstate__(self,state):
         super(QTTvec,self).__setstate__( state )
 
-    def build(self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, kickrank=None):
+    def build(self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, max_ranks=None, kickrank=None):
         """ Common interface for the construction of the approximation.
 
         :param float eps: [default == 1e-10] For method=='svd': precision with which to approximate the input tensor. For method=='ttcross': TT-rounding tolerance for rank-check.
-        :param string method: 'svd' use singular value decomposition to construct the TT representation [1]_, 'ttcross' use low rank skeleton approximation to construct the TT representation [2]_, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation [3]_ [4]_, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
+        :param string method: 'svd' use singular value decomposition to construct the TT representation :cite:`Oseledets2011`, 'ttcross' use low rank skeleton approximation to construct the TT representation :cite:`Oseledets2010`, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation :cite:`Savostyanov2011,Savostyanov2013`, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
         :param list rs: list of integer ranks of different cores. If ``None`` then the incremental TTcross approach will be used. (method=='ttcross')
         :param bool fix_rank: determines whether the rank is allowed to be increased (method=='ttcross')
         :param list Jinit: list of list of integers containing the r starting columns in the lowrankapprox routine for each core. If ``None`` then pick them randomly. (method=='ttcross')
@@ -161,16 +164,17 @@ class QTTvec(TTvec):
         :param float mv_eps: accuracy parameter for each usage of the maxvol algorithm (method=='ttcross')
         :param int mv_maxit: maximum number of iterations in the maxvol routine (method=='ttcross')
         :param bool fix_rank: Whether the rank is allowed to increase
+        :param list max_ranks: Maximum ranks to be used to limit the trunaction rank due to ``eps``. The first and last elements of the list must be ``1``, e.g. ``[1,...,1]``. Default: ``None``.
         :param int kickrank: rank overshooting for 'ttdmrg'
         """
         
         if isinstance(self.A,np.ndarray) or isinstance(self.A, TensorWrapper):
 
-            super(QTTvec,self).build( eps, method, rs, fix_rank, Jinit, delta, maxit, mv_eps, mv_maxit, kickrank)
+            super(QTTvec,self).build( eps=eps, method=method, rs=rs, fix_rank=fix_rank, Jinit=Jinit, delta=delta, maxit=maxit, mv_eps=mv_eps, mv_maxit=mv_maxit, max_ranks=max_ranks, kickrank=kickrank)
             
         elif isinstance(self.A,list):
             
-            super(QTTvec,self).build( eps, method, rs, fix_rank, Jinit, delta, maxit, mv_eps, mv_maxit, kickrank)
+            super(QTTvec,self).build( eps=eps, method=method, rs=rs, fix_rank=fix_rank, Jinit=Jinit, delta=delta, maxit=maxit, mv_eps=mv_eps, mv_maxit=mv_maxit, max_ranks=max_ranks, kickrank=kickrank)
 
             shape = self.shape()
 
@@ -205,6 +209,42 @@ class QTTvec(TTvec):
         for TTi in self.TT: newTT.append(TTi.copy())
         return QTTvec(newTT,list(self.global_shape)).build()
 
+    def get_data_F_norm(self):
+        """ Used to get the Frobeniuos norm of the underlying data.
+        
+        .. note: To get the Frobenious norm of the TT approximation, use the :py:method:`multilinalg.norm`
+        """
+        return npla.norm(self.A[tuple([ slice(0,gs,None) for gs in self.get_global_shape() ])].flatten(),2)
+
+    def get_ttdmrg_real_subtensor(self,C,idxs):
+        """ Used to get the Frobeniuos norm of a subtensor of the underlying data.
+  
+        :param np.ndarray C: Extracted 4-d tensor with shape len(l_idx) x n x m x len(r_idx)
+        :param list idxs: List of tuples of the form (l_idx, slice, slice, r_idx)
+
+        :return np.ndarray Creal: 1-d array containing the filtered values belonging to the real tensor
+        
+        .. note: To get the Frobenious norm of the TT approximation, use the :py:method:`multilinalg.norm`
+        """
+        # Expand the indices
+        (lidxs,_,_) = expand_idxs(idxs, self.shape())
+        
+        # Transpose C in order to be iterated the same way of the lidxs
+        Csh = np.transpose(C, (0,3,1,2))
+        
+        # Filter the redundant values
+        Creal = []
+        for i,(idx,(_,value)) in enumerate( zip(lidxs,np.ndenumerate(Csh)) ):
+            
+            # Get the index representation in the unfolded base q tensor
+            idx = self.full_to_q( idx )
+
+            if all( ii < si for (ii,si) in zip(idx,self.get_global_shape()) ):
+                Creal.append(value)
+        
+        return np.array(Creal)
+        # return C.flatten()
+
     def get_global_shape(self):
         """ Return the shape of the original tensor
         """
@@ -238,7 +278,7 @@ class QTTvec(TTvec):
         for subshape,gs in zip(self.folded_shape,self.get_global_shape()):
             tmpcore = self.TT[icore]
             icore += 1
-            for i in xrange(1,len(subshape)):
+            for i in range(1,len(subshape)):
                 tmpcore = np.tensordot( tmpcore, self.TT[icore], ( (tmpcore.ndim-1,),(0,) ) )
                 icore += 1
             tmpcore = np.reshape( tmpcore, 
@@ -266,7 +306,7 @@ class QTTvec(TTvec):
 
     def get_ttdmrg_eval_idxs(self):
         idxs_full = super(QTTvec,self).get_ttdmrg_eval_idxs()
-        idxs = [ self.full_to_global( idxs_full[i,:] ) for i in xrange(idxs_full.shape[0]) ]
+        idxs = [ self.full_to_global( idxs_full[i,:] ) for i in range(idxs_full.shape[0]) ]
         return np.asarray(idxs)
 
     def __getitem__(self,idxs):
@@ -292,7 +332,7 @@ class QTTvec(TTvec):
         self.folded_shape.extend(A.get_folded_shape())
     
     def interpolate(self, Ms=None,eps=1e-8,is_sparse=None):
-        """ Interpolates the values of the QTTvec at arbitrary points, using the interpolation matrices ``Ms''.
+        """ Interpolates the values of the QTTvec at arbitrary points, using the interpolation matrices ``Ms``.
         
         :param list Ms: list of interpolation matrices for each dimension. Ms[i].shape[1] == self.shape()[i]
         :param float eps: tolerance with which to perform the rounding after interpolation
@@ -332,7 +372,7 @@ class QTTvec(TTvec):
         return mla.dot(TT_MND,self).rounding(eps)
 
     def project(self, Vs=None, Ws=None, eps=1e-8,is_sparse=None):
-        """ Project the QTTvec onto a set of basis provided, using the Generalized Vandermonde matrices ``Vs'' and weights ``Ws''.
+        """ Project the QTTvec onto a set of basis provided, using the Generalized Vandermonde matrices ``Vs`` and weights ``Ws``.
         
         :param list Vs: list of generalized Vandermonde matrices for each dimension. Ms[i].shape[1] == self.shape()[i]
         :param list Ws: list of weights for each dimension. Ws[i].shape[0] == self.shape()[i]
@@ -399,7 +439,7 @@ def QTTzerosvec(d,N,base):
     from TensorToolbox.core import Candecomp
     from TensorToolbox.core import zerosvec
 
-    if isinstance(N,int):
+    if isint(N):
         N = [N for i in range(d)]
     
     for sizedim in N:
