@@ -15,7 +15,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -38,7 +38,7 @@ from scipy import linalg as scla
 from scipy import sparse as scsp
 from scipy.sparse import linalg as spla
 
-from TensorToolbox.core import TensorWrapper, TTvec, QTTvec, storable_object
+from TensorToolbox.core import TensorWrapper, TTvec, QTTvec, storable_object, isint, isfloat
 from TensorToolbox import multilinalg as mla
 
 from SpectralToolbox import Spectral1D as S1D
@@ -49,7 +49,7 @@ PROJECTION    = 'Projection'
 AVAIL_SURROGATE_TYPES = [ LINEAR_INTERPOLATION, LAGRANGE_INTERPOLATION, PROJECTION ]
 
 class STT(storable_object):
-    """ Constructor of the Spectral Tensor Train approximation. Given a function ``f(x,theta,params):(Is, It) -> R``
+    """ Constructor of the Spectral Tensor Train approximation :cite:`Bigoni2015`. Given a function ``f(x,theta,params):(Is, It) -> R``
     with ``dim(Is)=n`` and ``dim(It)=d``, construct an approximation of ``g(theta,params): It -> h_t(Is)``. For example ``Is`` could be the discretization of a spatial dimension, and ``It`` some parameter space, so that ``f(x,theta,params)`` describes a scalar field depending some parameters that vary in ``It``. The ``params`` in the definition of ``f`` can be constants used by the function or othere objects that must be passed to the function definition.
     
     :param function f: multidimensional function to be approximated with format ``f(x,theta,params)``
@@ -141,7 +141,7 @@ class STT(storable_object):
 
         # Parameters to be reset on restarting
         self.f = None                    # Function to be approximated
-        self.polys = None                # List of polynomials (Spectral1D.Poly1D)
+        self.polys = None                # List of polynomials (from Spectral1D.AVAIL_POLY)
         #
         # End list of attributes
         ##########################################################
@@ -208,7 +208,7 @@ class STT(storable_object):
         self.polys = [None for i in range(self.param_dim)]
         for i, (poly_type, poly_param) in enumerate(zip(self.poly_types,self.poly_params)):
             if poly_type != None:
-                self.polys[i] = S1D.Poly1D(poly_type,poly_param)
+                self.polys[i] = S1D.generate(poly_type,poly_param)
     
     def __getitem__(self):
         raise NotImplementedError("To be implemented! (Maybe it's better to just use __call__")
@@ -234,7 +234,7 @@ class STT(storable_object):
 
             Np = x.shape[0]
             
-            for i in xrange(self.param_dim):
+            for i in range(self.param_dim):
                 if not (np.inf in self.spans[i]):
                     # Check all the values are in the span range
                     if any(np.logical_or(x[:,i] < self.spans[i][0], x[:,i] > self.spans[i][1])):
@@ -245,17 +245,17 @@ class STT(storable_object):
             
             output = np.empty((Np,) + self.space_shape)
 
-            mat_cache = [ {} for i in xrange(self.param_dim) ]
-            MsI = [ None for i in xrange(self.param_dim) ]
+            mat_cache = [ {} for i in range(self.param_dim) ]
+            MsI = [ None for i in range(self.param_dim) ]
             
             if self.surr_type == PROJECTION: # Projection
                 for ((point,TTf),(_,orders)) in \
                         zip( np.ndenumerate(self.TTfour), np.ndenumerate(self.orders) ):
                     self.logger.debug("Computing point %s" % (str(point)))
-                    for i in xrange(self.param_dim):
+                    for i in range(self.param_dim):
                         try: MsI[i] = mat_cache[i][orders[i]]
                         except KeyError:
-                            mat_cache[i][orders[i]] = self.polys[i].GradVandermonde1D(x[:,i], orders[i], 0, norm=True)
+                            mat_cache[i][orders[i]] = self.polys[i].GradVandermonde(x[:,i], orders[i], 0, norm=True)
                             MsI[i] = mat_cache[i][orders[i]]
                     TTval = TTf.interpolate(MsI)
                     output[ (slice(None,None,None),) + point ] = np.asarray( [ TTval[tuple([i]*self.param_dim)] for i in range(Np) ] ) 
@@ -263,7 +263,7 @@ class STT(storable_object):
                 for ((point,TTapp),(_,Xs)) in \
                         zip( np.ndenumerate(self.TTapprox),np.ndenumerate(self.Xs_params) ):
                     self.logger.debug("Computing point %s" % (str(point)))
-                    for i in xrange(self.param_dim):
+                    for i in range(self.param_dim):
                         try: MsI[i] = mat_cache[i][tuple(Xs[i])]
                         except KeyError:
                             mat_cache[i][tuple(Xs[i])] = S1D.SparseLinearInterpolationMatrix(Xs[i],x[:,i]).tocsr()
@@ -275,7 +275,7 @@ class STT(storable_object):
                 for ((point,TTapp),(_,Xs),(_,bw)) in \
                         zip( np.ndenumerate(self.TTapprox),np.ndenumerate(self.Xs_params),np.ndenumerate(self.barycentric_weights) ):
                     self.logger.debug("Computing point %s" % (str(point)))
-                    for i in xrange(self.param_dim):
+                    for i in range(self.param_dim):
                         try: MsI[i] = mat_cache[i][tuple(Xs[i])]
                         except KeyError:
                             mat_cache[i][tuple(Xs[i])] = S1D.LagrangeInterpolationMatrix(Xs[i],bw[i],x[:,i])
@@ -311,7 +311,7 @@ class STT(storable_object):
     
     def __iadd__(A,B):
         if not ( (isinstance(A, STT) and A.init) and \
-                     ((isinstance(B,STT) and B.init) or isinstance(B, np.ndarray) or isinstance(B, float))): 
+                     ((isinstance(B,STT) and B.init) or isinstance(B, np.ndarray) or isfloat(B))): 
             raise RuntimeError("The STT are not initialized")
         if isinstance(A, STT) and isinstance(B, STT):
             for (point,TTa),(_,TTb) in \
@@ -323,7 +323,7 @@ class STT(storable_object):
                     zip( np.ndenumerate(A.TTapprox), np.ndenumerate(B) ):
                 A.TTapprox[point] = (TTa + b).rounding( A.eps )
 
-        elif isinstance(A, STT) and isinstance(B, float):
+        elif isinstance(A, STT) and isfloat(B):
             for (point,TTa) in np.ndenumerate(A.TTapprox):
                 A.TTapprox[point] = (TTa + B).rounding( A.eps )
 
@@ -356,14 +356,14 @@ class STT(storable_object):
     
     def __imul__(A,B):
         if not ( (isinstance(A, STT) and A.init) and \
-                     ((isinstance(B,STT) and B.init) or isinstance(B, np.ndarray) or isinstance(B, float))): 
+                     ((isinstance(B,STT) and B.init) or isinstance(B, np.ndarray) or isfloat(B))): 
             raise RuntimeError("The STT are not initialized")
         if isinstance(A,STT) and isinstance(B,STT):
             for (point,TTa),(_,TTb) in \
                 zip( np.ndenumerate(A.TTapprox), np.ndenumerate(B.TTapprox) ):
                 A.TTapprox[point] = (TTa * TTb).rounding( min(A.eps,B.eps) )
         
-        elif isinstance(B,float) and isinstance(A,STT):
+        elif isfloat(B) and isinstance(A,STT):
             for (point,TTa) in np.ndenumerate(A.TTapprox):
                 A.TTapprox[point] = (TTa * B).rounding( min(A.eps) )
 
@@ -377,7 +377,7 @@ class STT(storable_object):
     
     def __pow__(A,n):
         if not A.init: raise RuntimeError("The STT is not initialized")
-        if not isinstance(n,int): raise RuntimeError("n must be an integer")
+        if not isint(n): raise RuntimeError("n must be an integer")
         B = A.copy()
         for (point,TTb) in np.ndenumerate(B.TTapprox):
             B.TTapprox[point] = (TTb**n).rounding( B.eps )
@@ -426,11 +426,11 @@ class STT(storable_object):
         C.polys = [None for i in range(self.param_dim)]
         for i, (poly_type, poly_param) in enumerate(zip(self.poly_types,self.poly_params)):
             if poly_type != None:
-                C.polys[i] = S1D.Poly1D(poly_type,poly_param)
+                C.polys[i] = S1D.generate(poly_type,poly_param)
         return C
  
     def set_f(self,f,marshal_f=True):
-        if f == None:
+        if f is None:
             self.TW.reset_f_marshal()
         else:
             self.TW.set_f(f,marshal_f)
@@ -466,7 +466,7 @@ class STT(storable_object):
         self.orders = np.empty(self.space_shape, dtype=list)
         self.Vs = np.empty(self.space_shape, dtype=list)
         
-        if orders == None:
+        if orders is None:
             for p, value in np.ndenumerate(self.orders): self.orders[p] = [2 for i in range(self.param_dim)]
         else:
             if not (len(orders) == self.param_dim):
@@ -479,7 +479,7 @@ class STT(storable_object):
                 self.spans[i] = (grid[0], grid[-1])
             elif isinstance(grid,tuple) and len(grid)==4:
                 (self.poly_types[i],self.quad_types[i],self.poly_params[i],self.spans[i]) = grid
-                self.polys[i] = S1D.Poly1D(self.poly_types[i],self.poly_params[i])
+                self.polys[i] = S1D.generate(self.poly_types[i],self.poly_params[i])
             elif isinstance(grid,tuple) and len(grid)==2:
                 (self.quad_types[i],self.spans[i]) = grid
 
@@ -495,7 +495,7 @@ class STT(storable_object):
                 if isinstance(grid,np.ndarray): 
                     self.Xs_params[point][i] = grid
                 elif isinstance(grid,tuple) and len(grid)==4:
-                    (x,w) = self.polys[i].Quadrature(self.orders[point][i],quadType=self.quad_types[i], normed=False)
+                    (x,w) = self.polys[i].Quadrature(self.orders[point][i],quadType=self.quad_types[i], norm=False)
                     self.poly_Xs[point][i] = x.copy()
                     self.poly_Ws[point][i] = w.copy()
                     if not (np.inf in self.spans[i]):
@@ -529,12 +529,12 @@ class STT(storable_object):
     def build(self,maxprocs=None,force_redo=False):
         self.start_build_time = time.clock()
         
-        if self.generic_approx == None or force_redo:
+        if self.generic_approx is None or force_redo:
             self.empty_generic_approx()
         
         self.TW.set_maxprocs(maxprocs) # Set the number of processors for MPI
         for point, val in np.ndenumerate(self.generic_approx):
-            if val == None or not val.init:
+            if val is None or not val.init:
                 try:
                     totsize = float(self.TW.get_size())
                     self.logger.info( "Point %s - Fill: %f%%" % (str(point),float(self.TW.get_fill_level())/totsize * 100.) )
@@ -542,7 +542,7 @@ class STT(storable_object):
                     self.logger.info( "Point %s - Fill: %d" % (str(point),self.TW.get_fill_level()) )
                 multidim_point = point if self.range_dim > 0 else None
                 # Build generic_approx for the selected point
-                if val == None or self.generic_approx[point].Jinit == None:
+                if val is None or self.generic_approx[point].Jinit is None:
                     # Find all the back neighbors and select the first found to start from
                     neigh = None
                     for i in range(self.range_dim-1,-1,-1): 
@@ -607,7 +607,7 @@ class STT(storable_object):
     def prepare_TTapprox(self, force_redo=False):
         """ Prepares the TTapprox from the generic_approx
         """
-        if self.TTapprox == None or force_redo:
+        if self.TTapprox is None or force_redo:
             self.TTapprox = np.empty(self.space_shape, dtype=TTvec)
         
         for (point,gen_app) in np.ndenumerate(self.generic_approx):
@@ -625,23 +625,23 @@ class STT(storable_object):
     
     def build_projection_surrogate(self,force_redo=False):
         # Create the TT approximation of the Fourier coefficients (Project)
-        if self.TTfour == None or force_redo:
+        if self.TTfour is None or force_redo:
             self.TTfour = np.empty(self.space_shape, dtype=TTvec)
 
         for (point, val),(_,ttapp) in zip(np.ndenumerate(self.TTfour), np.ndenumerate(self.TTapprox)) :
-            if val == None and ttapp != None:
-                self.Vs[point] = [ self.polys[i].GradVandermonde1D(self.poly_Xs[point][i], self.orders[point][i], 0, norm=True) for i in range(self.param_dim) ]
+            if val is None and ttapp != None:
+                self.Vs[point] = [ self.polys[i].GradVandermonde(self.poly_Xs[point][i], self.orders[point][i], 0, norm=True) for i in range(self.param_dim) ]
                 self.logger.debug("Projecting point %s" % (str(point)))
                 self.TTfour[point] = ttapp.project(self.Vs[point],self.poly_Ws[point])
         self.logger.debug("Projection Done")
     
     def build_barycentric_weights(self, force_redo=False):
-        if self.barycentric_weights == None or force_redo:
+        if self.barycentric_weights is None or force_redo:
             self.barycentric_weights = np.empty(self.space_shape, dtype=list)
         
         for (point, _),(_,X) in \
                 zip( np.ndenumerate(self.barycentric_weights), np.ndenumerate(self.Xs_params) ):
-            self.barycentric_weights[point] = [ S1D.BarycentricWeights(X[i]) for i in xrange(self.param_dim) ]
+            self.barycentric_weights[point] = [ S1D.BarycentricWeights(X[i]) for i in range(self.param_dim) ]
 
     def integrate(self):
         """ Compute the integral of the approximated function
