@@ -15,7 +15,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -28,7 +28,7 @@ import logging
 import numpy as np
 from scipy import sparse as scsp
 
-from TensorToolbox.core import TTvec, mat_to_tt_idxs
+from TensorToolbox.core import TTvec, WTTvec, mat_to_tt_idxs, isint, isfloat
 
 class TTmat(TTvec):
     """ Constructor of multidimensional matrix in Tensor Train format
@@ -40,6 +40,7 @@ class TTmat(TTvec):
     :param list sparse_ranks: [default==None] mandatory argument when A is a list of sparse matrices. It contains integers listing the TT-ranks of the matrix.
 
     .. note:: the method __getitem__ is not overwritten, thus the indices used to access the tensor refer to the flatten versions of the matrices composing the matrix tensor.
+
     """
 
     logger = logging.getLogger(__name__)
@@ -77,11 +78,11 @@ class TTmat(TTvec):
     def __setstate__(self,state):
         super(TTmat,self).__setstate__( state )
 
-    def build(self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, kickrank=2):
+    def build(self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, max_ranks=None, kickrank=2):
         """ Common interface for the construction of the approximation.
 
         :param float eps: [default == 1e-10] For method=='svd': precision with which to approximate the input tensor. For method=='ttcross': TT-rounding tolerance for rank-check.
-        :param string method: 'svd' use singular value decomposition to construct the TT representation [1]_, 'ttcross' use low rank skeleton approximation to construct the TT representation [2]_, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation [3]_ [4]_, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
+        :param string method: 'svd' use singular value decomposition to construct the TT representation :cite:`Oseledets2011`, 'ttcross' use low rank skeleton approximation to construct the TT representation :cite:`Oseledets2010`, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation :cite:`Savostyanov2011,Savostyanov2013`, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
         :param list rs: list of integer ranks of different cores. If ``None`` then the incremental TTcross approach will be used. (method=='ttcross')
         :param bool fix_rank: determines whether the rank is allowed to be increased (method=='ttcross')
         :param list Jinit: list of list of integers containing the r starting columns in the lowrankapprox routine for each core. If ``None`` then pick them randomly. (method=='ttcross')
@@ -90,6 +91,7 @@ class TTmat(TTvec):
         :param float mv_eps: accuracy parameter for each usage of the maxvol algorithm (method=='ttcross')
         :param int mv_maxit: maximum number of iterations in the maxvol routine (method=='ttcross')
         :param bool fix_rank: Whether the rank is allowed to increase
+        :param list max_ranks: Maximum ranks to be used to limit the trunaction rank due to ``eps``. The first and last elements of the list must be ``1``, e.g. ``[1,...,1]``. Default: ``None``.
         :param int kickrank: rank overshooting for 'ttdmrg'
         """
         
@@ -102,7 +104,9 @@ class TTmat(TTvec):
         self.is_sparse = []
         self.sparse_TT = []
         
-        if isinstance(self.A,list) and np.all( [ isinstance(self.A[i],scsp.csr_matrix) for i in range(len(self.A)) ] ):
+        if isinstance(self.A,list) and np.all( [ (isinstance(self.A[i],scsp.csr_matrix) or
+                                                  isinstance(self.A[i],scsp.csc_matrix) or
+                                                  isinstance(self.A[i],scsp.dia_matrix)) for i in range(len(self.A)) ] ):
             if self.sparse_ranks == None:
                 raise AttributeError("The parameter sparse_ranks must be defined for only-sparse initialization")
             
@@ -112,7 +116,7 @@ class TTmat(TTvec):
             self.sparse_only = True
             self.sparse_TT = self.A
 
-            if isinstance(nrows,int) and isinstance(ncols,int):
+            if isint(nrows) and isin(ncols):
                 d = len(self.sparse_TT)
                 self.nrows = [nrows for i in range(d)]
                 self.ncols = [ncols for i in range(d)]
@@ -128,12 +132,14 @@ class TTmat(TTvec):
             
             self.init = True
 
-        elif isinstance(self.A,list) and np.any( [ isinstance(self.A[i],scsp.csr_matrix) for i in range(len(self.A)) ] ):
+        elif isinstance(self.A,list) and np.any( [ (isinstance(self.A[i],scsp.csr_matrix) or
+                                                    isinstance(self.A[i],scsp.csc_matrix) or
+                                                    isinstance(self.A[i],scsp.dia_matrix)) for i in range(len(self.A)) ] ):
             raise TypeError("Mixed sparse/full initialization not implemented yet")
         else:
             self.sparse_only = False
-            super(TTmat,self).build( eps, method, rs, fix_rank, Jinit, delta, maxit, mv_eps, mv_maxit, kickrank )
-            if isinstance(nrows,int) and isinstance(ncols,int):
+            super(TTmat,self).build( eps=eps, method=method, rs=rs, fix_rank=fix_rank, Jinit=Jinit, delta=delta, maxit=maxit, mv_eps=mv_eps, mv_maxit=mv_maxit, max_ranks=max_ranks, kickrank=kickrank )
+            if isint(nrows) and isint(ncols):
                 d = len(self.TT)
                 self.nrows = [nrows for i in range(d)]
                 self.ncols = [ncols for i in range(d)]
@@ -238,8 +244,11 @@ class TTmat(TTvec):
                 Yi = np.reshape(Yi_rsh,(Ai0*Bi.shape[0],self.nrows[i],Ai2*Bi.shape[2]))
 
                 Y.append(Yi)
-
-            return TTvec(Y).build()
+            
+            if isinstance(B,WTTvec):
+                return WTTvec(Y,B.sqrtW).build()
+            else:
+                return TTvec(Y).build()
 
         elif isinstance(B,TTmat):
             if not self.init or not B.init: raise NameError("TensorToolbox.TTmat.dot: TT not initialized correctly")
@@ -336,8 +345,8 @@ def randmat(d,nrows,ncols):
     import numpy.random as npr
     from TensorToolbox.core import Candecomp
 
-    if isinstance(nrows,int): nrows = [ nrows for i in range(d) ]
-    if isinstance(ncols,int): ncols = [ ncols for i in range(d) ]
+    if isint(nrows): nrows = [ nrows for i in range(d) ]
+    if isint(ncols): ncols = [ ncols for i in range(d) ]
     CPtmp = [npr.random(nrows[i]*ncols[i]).reshape((1,nrows[i]*ncols[i])) + 0.5 for i in range(d)]
     CP_rand = Candecomp(CPtmp)
     TT_rand = TTmat(CP_rand,nrows,ncols).build()
@@ -358,7 +367,7 @@ def eye(d,N):
        TODO: improve construction avoiding passage through Candecomp
     """
     from TensorToolbox.core import Candecomp
-    if isinstance(N, int):
+    if isint(N):
         If = np.eye(N).flatten().reshape((1,N**2))
         CPtmp = [If for i in range(d)]
     elif isinstance(N, list):
