@@ -18,7 +18,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -31,10 +31,14 @@ import logging
 import sys
 import warnings
 import itertools
+try:
+    import itertools.izip as zip
+except ImportError:
+    pass
 import copy
 import operator
 import random
-
+from functools import reduce
 
 import numpy as np
 import numpy.random as npr
@@ -44,31 +48,23 @@ from scipy import linalg as scla
 from scipy import sparse as scsp
 from scipy.sparse import linalg as spla
 
-from TensorToolbox.core import ConvergenceError,TTcrossLoopError,\
-    TensorWrapper,maxvol,idxfold,idxunfold,storable_object,\
-    Candecomp,reort
+from TensorToolbox.core import ConvergenceError, TTcrossLoopError,\
+    TensorWrapper, maxvol, idxfold, idxunfold, storable_object,\
+    Candecomp, reort, isint, isfloat
 from TensorToolbox import multilinalg as mla
 
 class TTvec(storable_object):
-    """ Constructor of multidimensional tensor in Tensor Train format [1]_
+    """ Constructor of multidimensional tensor in Tensor Train format :cite:`Oseledets2011`
         
-    :param Candecomp,ndarray,TT A: Available input formats are Candecomp, full tensor in numpy.ndarray, Tensor Train structure (list of cores)
+    :param Candecomp,ndarray,TT,TensorWrapper A: Available input formats are Candecomp, full tensor in numpy.ndarray, Tensor Train structure (list of cores), or a Tensor Wrapper.
     :param string store_location: Store computed values during construction on the specified file path. The stored values are ttcross_Jinit and the values used in the TensorWrapper. This permits a restart from already computed values. If empty string nothing is done. (method=='ttcross')
     :param string store_object: Object to be stored (default are the tensor wrapper and ttcross_Jinit)
     :param int store_freq: storage frequency. ``store_freq==1`` stores intermediate values at every iteration. The program stores data every ``store_freq`` internal iterations. If ``store_object`` is a SpectralTensorTrain, then ``store_freq`` determines the number of seconds every which to store values.
     :param int multidim_point: If the object A returns a multidimensional array, then this can be used to define which point to apply ttcross to.
     
-    References  
-    ----------
-
-    .. [1] Oseledets, I. (2011). Tensor-train decomposition. SIAM Journal on Scientific Computing, 33(5), 2295–2317. Retrieved from http://epubs.siam.org/doi/pdf/10.1137/090752286
-    .. [2] Oseledets, I., and Tyrtyshnikov, E. (2010). TT-cross approximation for multidimensional arrays. Linear Algebra and Its Applications, 432(1), 70-88. doi:10.1016/j.laa.2009.07.024
-    .. [3] Savostyanov, D., and Oseledets, I. (2011). Fast adaptive interpolation of multi-dimensional arrays in tensor train format. The 2011 International Workshop on Multidimensional (nD) Systems, 1–8. doi:10.1109/nDS.2011.6076873
-    .. [4] Savostyanov, D. (2013). Quasioptimality of maximum-volume cross interpolation of tensors. arXiv Preprint arXiv:1305.1818, c, 1–23. Retrieved from http://arxiv.org/abs/1305.1818
-
-
     .. document private functions
     .. automethod:: __getitem__
+    
     """
 
     logger = logging.getLogger(__name__)
@@ -92,6 +88,7 @@ class TTvec(storable_object):
         self.init = False
         self.method = 'svd'
         self.eps = None
+        self.max_ranks = None
         self.multidim_point = None
 
         # Values only stored for cross approximations (ttcross, ttdmrg)
@@ -109,7 +106,7 @@ class TTvec(storable_object):
         self.dmrg_sweep_direction = None
         self.dmrg_Pinv = None
 
-        self.serialize_list.extend( ['TT', 'init', 'method', 'eps', 'multidim_point', 'rs', 'Js', 'Is', 'Js_last', 'Jinit', 'ltor_fiber_lists', 'rtol_fiber_lists', 'dmrg_not_converged', 'dmrg_sweep_it', 'dmrg_sweep_direction', 'dmrg_Pinv'] )
+        self.serialize_list.extend( ['TT', 'init', 'method', 'eps', 'max_ranks', 'multidim_point', 'rs', 'Js', 'Is', 'Js_last', 'Jinit', 'ltor_fiber_lists', 'rtol_fiber_lists', 'dmrg_not_converged', 'dmrg_sweep_it', 'dmrg_sweep_direction', 'dmrg_Pinv'] )
         self.subserialize_list.extend( ['A'] )   # Not serialized thanks to the STT interface
         # End list of attributes
         ###############################
@@ -119,11 +116,11 @@ class TTvec(storable_object):
         self.A = A
         self.multidim_point = multidim_point
 
-    def build( self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, kickrank=None):
+    def build( self, eps=1e-10, method='svd', rs=None, fix_rank=False, Jinit=None, delta=1e-4, maxit=100, mv_eps=1e-6, mv_maxit=100, max_ranks=None, kickrank=None):
         """ Common interface for the construction of the approximation.
 
         :param float eps: [default == 1e-10] For method=='svd': precision with which to approximate the input tensor. For method=='ttcross': TT-rounding tolerance for rank-check.
-        :param string method: 'svd' use singular value decomposition to construct the TT representation [1]_, 'ttcross' use low rank skeleton approximation to construct the TT representation [2]_, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation [3]_ [4]_, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
+        :param string method: 'svd' use singular value decomposition to construct the TT representation :cite:`Oseledets2011`, 'ttcross' use low rank skeleton approximation to construct the TT representation :cite:`Oseledets2010`, 'ttdmrg' uses Tensor Train Renormalization Cross to construct the TT representation :cite:`Savostyanov2011,Savostyanov2013`, 'ttdmrgcross' uses 'ttdmrg' with 'ttcross' approximation of supercores
         :param list rs: list of integer ranks of different cores. If ``None`` then the incremental TTcross approach will be used. (method=='ttcross')
         :param bool fix_rank: determines whether the rank is allowed to be increased (method=='ttcross')
         :param list Jinit: list of list of integers containing the r starting columns in the lowrankapprox routine for each core. If ``None`` then pick them randomly. (method=='ttcross')
@@ -132,18 +129,21 @@ class TTvec(storable_object):
         :param float mv_eps: accuracy parameter for each usage of the maxvol algorithm (method=='ttcross')
         :param int mv_maxit: maximum number of iterations in the maxvol routine (method=='ttcross')
         :param bool fix_rank: Whether the rank is allowed to increase
+        :param list max_ranks: Maximum ranks to be used to limit the trunaction rank due to ``eps``. The first and last elements of the list must be ``1``, e.g. ``[1,...,1]``. Default: ``None``.
         :param int kickrank: rank overshooting for 'ttdmrg'
         """
 
         if isinstance(self.A,Candecomp):
             self.eps = eps
+            self.max_ranks = max_ranks
             self.TT = self.A.to_TT()
             self.init = True
-            self.rounding(self.eps)
+            self.rounding(self.eps,self.max_ranks)
         elif isinstance(self.A,np.ndarray) or isinstance(self.A,TensorWrapper):
             self.eps = eps
+            self.max_ranks = max_ranks
             if method == 'svd' or method == None: 
-                self.svd(self.eps)
+                self.svd(self.eps, self.max_ranks)
             elif method == 'ttcross': 
                 self.ttcross(self.eps,rs,Jinit,delta,maxit,mv_eps,mv_maxit,fix_rank)
             elif method == 'ttdmrg':
@@ -170,7 +170,7 @@ class TTvec(storable_object):
     def __getitem__(self,idxs):
         if not self.init: raise NameError("TensorToolbox.TTvec.__getitem__: TT not initialized correctly")
         # TT format: TT[i][idx] = core (matrix or row/col vector)
-        if isinstance(idxs,int):
+        if isint(idxs):
             return self.TT[0][:,idxs,:]
         else:
             out = np.array([1.])
@@ -207,6 +207,24 @@ class TTvec(storable_object):
         T = np.tensordot(T,np.array([1.]),((T_ax,),(0,)))
         return T
 
+    def get_data_F_norm(self):
+        """ Used to get the Frobeniuos norm of the underlying data.
+        This needs to be redefined in QTTvec in order to get the Frobeniuous norm of the real tensor.
+        
+        .. note: To get the Frobenious norm of the TT approximation, use the :py:method:`multilinalg.norm`
+        """
+        if isinstance(self.A, TensorWrapper):
+            nrm = npla.norm(self.A[ tuple([ slice(None,None,None) ] * self.A.get_ndim()) ].flatten(),2)
+        else:
+            nrm = npla.norm(self.A.flatten(),2)
+        return nrm
+
+    def get_ttdmrg_real_subtensor(self,C,idx):
+        """ Used to get the real subtensor of the underlying data.
+        This needs to be redefined in QTTvec in order to get the subtensor of the real tensor.
+        """
+        return C
+
     def get_ttcross_eval_idxs(self):
         idxs = []
         dims = self.shape()
@@ -248,10 +266,10 @@ class TTvec(storable_object):
         In place addition
         """
         if not ( (isinstance(A, TTvec) and A.init) and \
-                     ((isinstance(B,TTvec) and B.init) or isinstance(B, float)) ): 
+                     ((isinstance(B,TTvec) and B.init) or isfloat(B)) ): 
             raise NameError("TensorToolbox.TTvec.add: TT not initialized correctly")
 
-        if isinstance(A,TTvec) and isinstance(B,float):
+        if isinstance(A,TTvec) and isfloat(B):
             lscp = [ np.ones((1,sh)) for i,sh in enumerate(A.shape()) ]
             lscp[0] *= B
             CP = Candecomp( lscp )
@@ -271,6 +289,10 @@ class TTvec(storable_object):
                 A.TT[i] = tmpi
 
         return A
+
+    def __radd__(A,B):
+        B += A
+        return B
     
     def __neg__(A):
         B = -1. * A
@@ -286,10 +308,29 @@ class TTvec(storable_object):
         A += -B
         return A
 
-    def __div__(A,B):
-        if not isinstance(B,float):
+    def __rsub__(A,B):
+        B += -A
+        return B
+
+    def __truediv__(A,B): # Python3
+        if not isfloat(B):
             raise AttributeError("TensorToolbox.TTvec.div: Division implemented only for floats")
-        return (A * (1./B))
+        C = A.copy()
+        C *= 1./B
+        return C
+
+    def __div__(A,B):
+        if not isfloat(B):
+            raise AttributeError("TensorToolbox.TTvec.div: Division implemented only for floats")
+        C = A.copy()
+        C *= 1./B
+        return C
+
+    def __idiv__(A,B):
+        if not isfloat(B):
+            raise AttributeError("TensorToolbox.TTvec.div: Division implemented only for floats")
+        A *= 1./B
+        return A
         
     def __mul__(A,B):
         """
@@ -327,7 +368,7 @@ class TTvec(storable_object):
                 A.TT[i] = tmpi
             return A
 
-        elif isinstance(B,float) and isinstance(A,TTvec):
+        elif isfloat(B) and isinstance(A,TTvec):
             if not A.init:
                 raise NameError("TensorToolbox.TTvec.mul: TT not initialized correctly")
 
@@ -341,7 +382,7 @@ class TTvec(storable_object):
         if not A.init:
             raise NameError("TensorToolbox.TTvec.pow: TT not initialized correctly")
         
-        if isinstance(n,int):
+        if isint(n):
             B = A.copy()
             for i in range(n-1): B *= A
             return B
@@ -384,16 +425,26 @@ class TTvec(storable_object):
     #########################################
     # Construction
     #########################################
-    def svd(self, eps):
+    def svd(self, eps, max_ranks=None):
 
         self.method = 'svd'
         self.eps = eps
+        self.max_ranks = max_ranks
         
         """ TT-SVD """
         d = self.A.ndim
         n = self.A.shape
-        delta = (eps/max(np.sqrt(d-1),1e-300)) * npla.norm(self.A.flatten(),2) # Truncation parameter
-        C = self.A.copy()  # Temporary tensor
+        delta = (eps/max(np.sqrt(d-1),1e-300)) * self.get_data_F_norm() # Truncation parameter
+        
+        if self.max_ranks == None:
+            self.max_ranks = [ min( reduce(operator.mul,n[:i+1]), reduce(operator.mul,n[i+1:]) ) for i in range(d-1) ]
+            self.max_ranks.insert(0,1)
+            self.max_ranks.append(1)
+
+        if isinstance(self.A, TensorWrapper):
+            C = self.A[ tuple([ slice(None,None,None) ] * self.A.get_ndim()) ]
+        else:
+            C = self.A.copy()  # Temporary tensor
         G = []
         if d == 1:
             G.append( C.reshape(1,n[0],1) )
@@ -410,7 +461,10 @@ class TTvec(storable_object):
             # Compute the delta-rank of C
             CF = npla.norm(S,2)
             rk = 1
-            while npla.norm(S[rk:],2) > delta: rk += 1
+            while npla.norm(S[rk:],2) > delta and rk < self.max_ranks[k+1]: rk += 1
+
+            if rk == self.max_ranks[k+1] and npla.norm(S[rk:],2) > delta:
+                self.logger.warning("MaxRank truncation. Cores %i-%i. Required accuracy delta=%e, accuracy met: %e" % (k,k+1,delta,npla.norm(S[rk:],2)) )
 
             r[k+1] = rk
 
@@ -421,7 +475,7 @@ class TTvec(storable_object):
         self.TT = G
         self.init = True
 
-    def ttdmrgcross(self, eps, Jinit, maxit, mv_eps, mv_maxit, kickrank=None, store_init=True):
+    def ttdmrgcross(self, eps, Jinit, maxit, mv_eps, mv_maxit, kickrank=None, store_init=True, loop_detection=False):
         """ Construct a TT representation of A using TT-Density matrix renormalization group
         
         :param float eps: Frobenious tolerance of the approximation
@@ -432,6 +486,7 @@ class TTvec(storable_object):
         :param int kickrank: rank overshooting
         :param bool cross: if True it uses ttcross for the supercores. If False it uses plain SVD.
         :param bool store_init: whether to store the initial indices used (restarting from the same indices will lead to the same construction).
+        :param bool loop_detection: whether to check for loops. (Never occurred that we needed it)
 
         .. note: eps is divided by 2 for convergence reasons.
         """
@@ -439,7 +494,7 @@ class TTvec(storable_object):
                     kickrank=kickrank, cross=True, 
                     store_init=store_init)
 
-    def ttdmrg(self, eps, Jinit, maxit, mv_eps, mv_maxit, kickrank=None, cross=False, store_init=True):
+    def ttdmrg(self, eps, Jinit, maxit, mv_eps, mv_maxit, kickrank=None, cross=False, store_init=True, loop_detection=False):
         """ Construct a TT representation of A using TT-Density matrix renormalization group
         
         :param float eps: Frobenious tolerance of the approximation
@@ -450,6 +505,7 @@ class TTvec(storable_object):
         :param int kickrank: rank overshooting
         :param bool cross: if True it uses ttcross for the supercores. If False it uses plain SVD.
         :param bool store_init: whether to store the initial indices used (restarting from the same indices will lead to the same construction).
+        :param bool loop_detection: whether to check for loops. (Never occurred that we needed it)
 
         .. note: eps is divided by 2 for convergence reasons.
         """
@@ -506,7 +562,7 @@ class TTvec(storable_object):
                 self.TT[k-1] = np.dot(Q,QsqInv).T.reshape( (rs[k-1], n[k-1], rs[k]) )
 
             # Add the last core
-            idx = (slice(None,None,None),) + tuple(itertools.izip(*Js[0]))
+            idx = (slice(None,None,None),) + tuple(zip(*Js[0]))
             if self.multidim_point == None:
                 C = self.A[ idx ]
             else:
@@ -535,7 +591,7 @@ class TTvec(storable_object):
             r1 = np.eye(1)
             self.Js = [None] * d
             self.Js[-1] = [()]
-            for k in xrange(d,1,-1):
+            for k in range(d,1,-1):
                 self.TT[k-1] = np.tensordot( self.TT[k-1], r1, ((2,),(0,)) )
                 C = np.reshape(self.TT[k-1], ( rs[k-1], n[k-1] * rs[k] ) ).T
 
@@ -564,8 +620,11 @@ class TTvec(storable_object):
         it = 0
         stored_not_converged = self.dmrg_not_converged
         self.dmrg_not_converged = True
-        if self.ltor_fiber_lists == None: self.ltor_fiber_lists = []
-        if self.rtol_fiber_lists == None: self.rtol_fiber_lists = []
+
+        if loop_detection:
+            if self.ltor_fiber_lists == None: self.ltor_fiber_lists = []
+            if self.rtol_fiber_lists == None: self.rtol_fiber_lists = []
+
         while it < maxit and self.dmrg_not_converged:
             it += 1
             if it == 1 and stored_not_converged != None:
@@ -586,26 +645,29 @@ class TTvec(storable_object):
             # Left - right sweep 
             # The following condition is used for exact restarting purposes
             if it == 1 and self.dmrg_sweep_direction == 'lr':
-                drange = xrange(self.dmrg_sweep_it,d-1)
+                drange = range(self.dmrg_sweep_it,d-1)
             elif it > 1 or (it == 1 and self.dmrg_sweep_direction == None):
-                drange = xrange(d-1)
+                drange = range(d-1)
                 self.Is = [[()]]
                 self.dmrg_sweep_direction = 'lr'
 
-            ltor_fiber_list = [] # This might not be consistent on restart (but who cares??)
+            if loop_detection:
+                ltor_fiber_list = [] # This might not be consistent on restart (but who cares??)
+
             if self.dmrg_sweep_direction == 'lr':
                 for self.dmrg_sweep_it in drange:
-                    (flist, \
-                     self.dmrg_Pinv, \
-                     self.dmrg_not_converged) = self.__left_right_ttdmrg_step(it, 
-                                                                    self.dmrg_sweep_it, 
-                                                                    self.dmrg_not_converged,
-                                                                    self.dmrg_Pinv, 
-                                                                    kickrank,
-                                                                    mv_eps, 
-                                                                    mv_maxit, 
+                    ( flist, \
+                          self.dmrg_Pinv, \
+                          self.dmrg_not_converged ) = self.__left_right_ttdmrg_step(it, \
+                                                                    self.dmrg_sweep_it, \
+                                                                    self.dmrg_not_converged, \
+                                                                    self.dmrg_Pinv, \
+                                                                    kickrank, \
+                                                                    mv_eps, \
+                                                                    mv_maxit, \
                                                                     cross)
-                    ltor_fiber_list.extend(flist)
+                    if loop_detection:
+                        ltor_fiber_list.extend(flist)
             
             # End left-right sweep
             ################################
@@ -614,43 +676,46 @@ class TTvec(storable_object):
             # Right - left sweep
             # The following condition is used for exact restarting purposes
             if it == 1 and self.dmrg_sweep_direction == 'rl':
-                drange = xrange(self.dmrg_sweep_it,0,-1)
+                drange = range(self.dmrg_sweep_it,0,-1)
             else:
-                drange = xrange(d-1,0,-1)
+                drange = range(d-1,0,-1)
                 self.Js = [None] * d
                 self.Js[-1] = [()]
                 self.dmrg_sweep_direction = 'rl'
 
-            rtol_fiber_list = []
+            if loop_detection:
+                rtol_fiber_list = []
             if self.dmrg_sweep_direction == 'rl':
                 for self.dmrg_sweep_it in drange:
-                    (flist, 
-                     self.dmrg_Pinv, 
-                     self.dmrg_not_converged) = self.__right_left_ttdmrg_step(it, 
-                                                                    self.dmrg_sweep_it, 
-                                                                    self.dmrg_not_converged, 
-                                                                    self.dmrg_Pinv, 
-                                                                    kickrank, 
-                                                                    mv_eps, 
-                                                                    mv_maxit,
+                    (flist, \
+                         self.dmrg_Pinv, \
+                         self.dmrg_not_converged) = self.__right_left_ttdmrg_step(it, \
+                                                                    self.dmrg_sweep_it, \
+                                                                    self.dmrg_not_converged, \
+                                                                    self.dmrg_Pinv, \
+                                                                    kickrank, \
+                                                                    mv_eps, \
+                                                                    mv_maxit, \
                                                                     cross)
-                    rtol_fiber_list.extend(flist)
+                    if loop_detection:
+                        rtol_fiber_list.extend(flist)
             
             # End right-left sweep
             ###############################
             
-            loop_detected = False
-            i = 0
-            while (not loop_detected) and i < len(self.rtol_fiber_lists)-1:
-                loop_detected = all(map( operator.eq, self.rtol_fiber_lists[i], rtol_fiber_list )) \
-                    and all(map( operator.eq, self.ltor_fiber_lists[i], ltor_fiber_list ))
-                i += 1
-            
-            self.ltor_fiber_lists.append(ltor_fiber_list)
-            self.rtol_fiber_lists.append(rtol_fiber_list)
-            
-            if loop_detected:
-                self.logger.warning( "Loop detected!" )
+            if loop_detection:
+                loop_detected = False
+                i = 0
+                while (not loop_detected) and i < len(self.rtol_fiber_lists)-1:
+                    loop_detected = all(map( operator.eq, self.rtol_fiber_lists[i], rtol_fiber_list )) \
+                        and all(map( operator.eq, self.ltor_fiber_lists[i], ltor_fiber_list ))
+                    i += 1
+
+                self.ltor_fiber_lists.append(ltor_fiber_list)
+                self.rtol_fiber_lists.append(rtol_fiber_list)
+
+                if loop_detected:
+                    self.logger.warning( "Loop detected!" )
         
         if it >= maxit:
             raise ConvergenceError('Maximum number of iterations reached.')
@@ -704,14 +769,27 @@ class TTvec(storable_object):
         for ii,jj in itertools.product(self.Is[k],self.Js[k+1]):
             fiber = ii + (slice(None,None,None), slice(None,None,None)) + jj
             flist.append(fiber)
-        
+
+        # Create index list to get the values at once
+        if k == 0:
+            idx = (slice(None,None,None), slice(None,None,None)) + tuple( zip(*self.Js[k+1]) )
+        else:
+            it = itertools.product(self.Is[k],self.Js[k+1])
+            idx = [ [] for i in range(d) ]
+            for (lidx,ridx) in it:
+                for j,jj in enumerate(lidx): idx[j].append(jj)
+                for j,jj in enumerate(ridx): idx[len(lidx)+2+j].append(jj)
+            idx[k] = slice(None,None,None)
+            idx[k+1] = slice(None,None,None)
+            idx = tuple(idx)
+
         if cross:
             # For each fiber construct a TTvec with ttcross on the filtered TensorWrapper
             C = []
             for ii,jj in itertools.product(self.Is[k],self.Js[k+1]):
                 if isinstance(self.A, TensorWrapper):
                     fix_idxs = list( ii + jj )
-                    fix_dims = range(len(ii)) + range(len(ii)+2,d)
+                    fix_dims = list(range(len(ii))) + list(range(len(ii)+2,d))
                     self.A.fix_indices(fix_idxs,fix_dims)
                     TTapp = TTvec(self.A, 
                                   store_object=self.store_object,
@@ -731,19 +809,6 @@ class TTvec(storable_object):
             C = np.reshape(C, ( len(self.Is[k]), len(self.Js[k+1]), n[k], n[k+1] ) )
             C = C.transpose( (0,2,3,1) )
         else:
-            # Create index list to get the values at once
-            if k == 0:
-                idx = (slice(None,None,None), slice(None,None,None)) + tuple( itertools.izip(*self.Js[k+1]) )
-            else:
-                it = itertools.product(self.Is[k],self.Js[k+1])
-                idx = [ [] for i in range(d) ]
-                for (lidx,ridx) in it:
-                    for j,jj in enumerate(lidx): idx[j].append(jj)
-                    for j,jj in enumerate(ridx): idx[len(lidx)+2+j].append(jj)
-                idx[k] = slice(None,None,None)
-                idx[k+1] = slice(None,None,None)
-                idx = tuple(idx)
-            
             # Extract values
             C = np.asarray(self.A[ idx ].tolist())
             mdim_point = () if self.multidim_point == None else self.multidim_point
@@ -774,7 +839,17 @@ class TTvec(storable_object):
         B = np.tensordot( self.TT[k], self.TT[k+1], ((2,),(0,)) )
         B = np.tensordot( Pinv[k], B, ( (1,),(0,) ) )
         B = np.tensordot( B, Pinv[k+2], ( (3,),(0,) ) )
-        if npla.norm( (C-B).flatten(), 2 ) > self.eps * npla.norm( C.flatten(), 2 ):
+
+        # if npla.norm( (C-B).flatten(), 2 ) > self.eps * npla.norm(C.flatten(),2):
+        #     not_converged = True
+
+        # Here the Frobenious norm is taken over only the entries belonging to the real tensor,
+        # not to any other extended tensor (e.g. QTT extended). This applies only for the right hand
+        # side, because on the left handside the difference will be 0 for the extended part of the
+        # tensor (by Linear Dependency of the extensions).
+        C_real = self.get_ttdmrg_real_subtensor(C,idx)
+        Diff_real = self.get_ttdmrg_real_subtensor(C-B,idx)
+        if npla.norm( Diff_real.flatten(), 2 ) > self.eps * npla.norm(C_real.flatten(),2):
             not_converged = True
 
         ###########################################
@@ -893,13 +968,26 @@ class TTvec(storable_object):
                 fiber = ii + (slice(None,None,None), slice(None,None,None)) + jj
                 flist.append(fiber)
 
+        # Create index list to get the values at once
+        if k == d-1:
+            idx = tuple( zip(*self.Is[k-1]) ) + (slice(None,None,None), slice(None,None,None))
+        else:
+            it = itertools.product(self.Is[k-1],self.Js[k])
+            idx = [ [] for i in range(d) ]
+            for (lidx,ridx) in it:
+                for j,jj in enumerate(lidx): idx[j].append(jj)
+                for j,jj in enumerate(ridx): idx[len(lidx)+2+j].append(jj)
+            idx[k-1] = slice(None,None,None)
+            idx[k] = slice(None,None,None)
+            idx = tuple(idx)
+
         if cross:
             # For each fiber construct a TTvec with ttcross on the filtered TensorWrapper
             C = []
             for ii,jj in itertools.product(self.Is[k-1],self.Js[k]):
                 if isinstance(self.A, TensorWrapper):
                     fix_idxs = list( ii + jj )
-                    fix_dims = range(len(ii)) + range(len(ii)+2,d)
+                    fix_dims = list(range(len(ii))) + list(range(len(ii)+2,d))
                     self.A.fix_indices(fix_idxs,fix_dims)
                     TTapp = TTvec(self.A, 
                                   store_object=self.store_object,
@@ -919,19 +1007,6 @@ class TTvec(storable_object):
             C = np.reshape(C, ( len(self.Is[k-1]), len(self.Js[k]), n[k-1], n[k] ) )
             C = C.transpose( (0,2,3,1) )
         else:
-            # Create index list to get the values at once
-            if k == d-1:
-                idx = tuple( itertools.izip(*self.Is[k-1]) ) + (slice(None,None,None), slice(None,None,None))
-            else:
-                it = itertools.product(self.Is[k-1],self.Js[k])
-                idx = [ [] for i in range(d) ]
-                for (lidx,ridx) in it:
-                    for j,jj in enumerate(lidx): idx[j].append(jj)
-                    for j,jj in enumerate(ridx): idx[len(lidx)+2+j].append(jj)
-                idx[k-1] = slice(None,None,None)
-                idx[k] = slice(None,None,None)
-                idx = tuple(idx)
-
             # Extract values        
             C = np.asarray(self.A[ idx ].tolist())
             mdim_point = () if self.multidim_point == None else self.multidim_point
@@ -962,7 +1037,17 @@ class TTvec(storable_object):
         B = np.tensordot( self.TT[k-1], self.TT[k], ((2,),(0,)) )
         B = np.tensordot( Pinv[k-1], B, ( (1,),(0,) ) )
         B = np.tensordot( B, Pinv[k+1], ( (3,),(0,) ) )
-        if npla.norm( (C-B).flatten(), 2 ) > self.eps * npla.norm( C.flatten(), 2 ):
+
+        # if npla.norm( (C-B).flatten(), 2 ) > self.eps * npla.norm(C.flatten(),2):
+        #     not_converged = True        
+
+        # Here the Frobenious norm is taken over only the entries belonging to the real tensor,
+        # not to any other extended tensor (e.g. QTT extended). This applies only for the right hand
+        # side, because on the left handside the difference will be 0 for the extended part of the
+        # tensor (by Linear Dependency of the extensions).
+        C_real = self.get_ttdmrg_real_subtensor(C,idx)
+        Diff_real = self.get_ttdmrg_real_subtensor(C-B,idx)
+        if npla.norm( Diff_real.flatten(), 2 ) > self.eps * npla.norm(C_real.flatten(),2):
             not_converged = True
         
         ###########################################
@@ -1261,7 +1346,7 @@ class TTvec(storable_object):
                 Gnew[k-1] = np.dot(Q,QsqInv).T.reshape( (rs[k-1], n[k-1], rs[k]) )
 
             # Add the last core
-            idx = (slice(None,None,None),) + tuple(itertools.izip(*Js[0]))
+            idx = (slice(None,None,None),) + tuple(zip(*Js[0]))
             if self.multidim_point == None:
                 C = self.A[ idx ]
             else:
@@ -1332,7 +1417,7 @@ class TTvec(storable_object):
             raise AttributeError("List of init indexes must be of length A.ndim-1")
         for k_Js in range(len(Jinit)-1):
             if Jinit[k_Js] == None:
-                if rs[k_Js+1] > max( np.prod(n[k_Js+1:]), sys.maxint ):
+                if rs[k_Js+1] > np.prod(n[k_Js+1:]):
                     raise ValueError("Ranks selected exceed the dimension of the tensor")
                 
                 # Lazy selection of indices... this can be done better.
@@ -1390,7 +1475,7 @@ class TTvec(storable_object):
                 flist.append(fiber)
 
         if k == 0:      # Is[k] will be empty
-            idx = (slice(None,None,None),) + tuple(itertools.izip(*Js[k]))
+            idx = (slice(None,None,None),) + tuple(zip(*Js[k]))
         else:
             it = itertools.product(Is[k],Js[k])
             idx = [ [] for i in range(d) ]
@@ -1462,7 +1547,7 @@ class TTvec(storable_object):
                 flist.append(fiber)
 
         if k == d:      # Is[k] will be empty
-            idx = tuple(itertools.izip(*Is[k-1])) + (slice(None,None,None),)
+            idx = tuple(zip(*Is[k-1])) + (slice(None,None,None),)
         else:
             it = itertools.product(Is[k-1],Js[k-1])
             idx = [ [] for i in range(d) ]
@@ -1500,11 +1585,17 @@ class TTvec(storable_object):
         if not A.init: raise NameError("TensorToolbox.TTvec.extend: input tensor is not initialized correctly")
         self.TT.extend(A.TT)
 
-    def rounding2(self,eps):
+    def rounding2(self,eps,max_ranks=None):
         """ TT-rounding """
         d = len(self.TT)
         n = self.shape()
         r = self.ranks()
+
+        if max_ranks == None:
+            max_ranks = [ min( reduce(operator.mul,n[:i+1]), reduce(operator.mul,n[i+1:]) ) for i in range(d-1) ]
+            max_ranks.insert(0,1)
+            max_ranks.append(1)
+        
         nrm = np.zeros(d,dtype=np.float64)
         core0 = self.TT[0]
         for i in range(d-1):
@@ -1526,6 +1617,9 @@ class TTvec(storable_object):
             core0 = np.reshape(core0,(r[i],n[i]*r[i+1]))
             (U,S,V) = scla.svd(core0,full_matrices=False)
             r1 = self.__round_chop(S,npla.norm(S,2)*ep)            # Truncate
+            if r1 > max_ranks[i]:
+                r1 = max_ranks[i]
+                self.logger.warning("MaxRank truncation. Cores %i-%i. Required accuracy delta=%e, accuracy met: %e" % (i,i-1,ep,npla.norm(S[r1:],2)) )
             U = U[:,:r1]
             S = S[:r1]
             V = V[:r1,:]
@@ -1557,11 +1651,16 @@ class TTvec(storable_object):
         ss = np.cumsum(S[::-1]**2.)
         return len(S) - next(i for i,s in enumerate(ss) if s > eps**2. or i == len(S)-1)
 
-    def rounding(self,eps):
+    def rounding(self,eps,max_ranks=None):
         """ TT-rounding """
         d = len(self.TT)
         ns = self.shape()
 
+        if max_ranks == None:
+            max_ranks = [ min( reduce(operator.mul,ns[:i+1]), reduce(operator.mul,ns[i+1:]) ) for i in range(d-1) ]
+            max_ranks.insert(0,1)
+            max_ranks.append(1)
+
         # OBS: The truncation parameter could be computed during the right to left orthogonalization?
         delta = eps/np.sqrt(d-1) # Truncation parameter
 
@@ -1591,6 +1690,9 @@ class TTvec(storable_object):
             # Truncate SVD
             rk = 0
             rk = self.__round_chop(S,npla.norm(S,2)*delta)            # Truncate
+            if rk > max_ranks[k+1]:
+                rk = max_ranks[k+1]
+                self.logger.warning("MaxRank truncation. Cores %i-%i. Required accuracy delta=%e, accuracy met: %e" % (k,k+1,delta,npla.norm(S[rk:],2)) )
 
             r.append( rk )
             self.TT[k] = U[:,:r[k+1]].reshape((r[k],self.TT[k].shape[1],r[k+1]))
@@ -1619,7 +1721,7 @@ class TTvec(storable_object):
         return self
 
     def interpolate(self,Ms=None,eps=1e-8,is_sparse=None):
-        """ Interpolates the values of the TTvec at arbitrary points, using the interpolation matrices ``Ms''.
+        """ Interpolates the values of the TTvec at arbitrary points, using the interpolation matrices ``Ms``.
         
         :param list Ms: list of interpolation matrices for each dimension. Ms[i].shape[1] == self.shape()[i]
         :param float eps: tolerance with which to perform the rounding after interpolation
@@ -1632,6 +1734,7 @@ class TTvec(storable_object):
         >>> Ms = [ S1D.LinearInterpolationMatrix(X[i],XI[i]) for i in range(d) ]
         >>> is_sparse = [True]*d
         >>> TTapproxI = TTapprox.interpolate(Ms,eps=1e-8,is_sparse=is_sparse)
+        
         """
         from TensorToolbox import TTmat
 
@@ -1662,7 +1765,7 @@ class TTvec(storable_object):
         return mla.dot(TT_MND,self).rounding(eps)
     
     def project(self, Vs=None, Ws=None, eps=1e-8,is_sparse=None):
-        """ Project the TTvec onto a set of basis provided, using the Generalized Vandermonde matrices ``Vs'' and weights ``Ws''.
+        """ Project the TTvec onto a set of basis provided, using the Generalized Vandermonde matrices ``Vs`` and weights ``Ws``.
         
         :param list Vs: list of generalized Vandermonde matrices for each dimension. Ms[i].shape[1] == self.shape()[i]
         :param list Ws: list of weights for each dimension. Ws[i].shape[0] == self.shape()[i]
@@ -1724,7 +1827,7 @@ def randvec(d,N):
     """
     from TensorToolbox.core import Candecomp
 
-    if isinstance(N,int):
+    if isint(N):
         N = [ N for i in range(d) ]
     CPtmp = [npr.random(N[i]).reshape((1,N[i])) + 0.5 for i in range(d)]
     CP_rand = Candecomp(CPtmp)
@@ -1744,7 +1847,7 @@ def zerosvec(d,N):
     """
     from TensorToolbox.core import Candecomp
 
-    if isinstance(N,int):
+    if isint(N):
         N = [ N for i in range(d) ]
     CPtmp = [np.zeros((1,N[i])) for i in range(d)]
     CP_zeros = Candecomp(CPtmp)
@@ -1764,7 +1867,7 @@ def onesvec(d,N):
     """
     from TensorToolbox.core import Candecomp
 
-    if isinstance(N,int):
+    if isint(N):
         N = [ N for i in range(d) ]
     CPtmp = [np.ones((1,N[i])) for i in range(d)]
     CP_ones = Candecomp(CPtmp)
