@@ -19,14 +19,14 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
 # Author: Daniele Bigoni
 #
 
-from multilinalg import *
+from TensorToolbox.multilinalg.multilinalg import *
 
 __author__ = "Daniele Bigoni"
 __copyright__ = """Copyright 2014, The Technical University of Denmark"""
