@@ -15,7 +15,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -103,20 +103,18 @@ def contraction(A,U):
 
 def norm(A,ord='fro',round_eps=1e-10,eps=1e-4,maxit=1000,pow_guess=None,info=False):
     """
-    Compute the norm of tensor A
+    Compute the norm of tensor A.
 
     Syntax:
         ``w = norm(A,[ord])``
 
     :param A: Tensor in some form
-    :param ord: Specifies the type of norm that needs to be computed. Available norms are
-                * Frobenius norm: 'fro'
+    :param ord: Specifies the type of norm that needs to be computed. Available norms are: the Frobenius norm: 'fro'. If the input tensor is a WeightedTensorTrainVec, then this takes the Frobenious norm of the weighted TT, i.e. the continuos norm defined by the weights.
     """
-    from TensorToolbox.core import TTvec
-    from TensorToolbox.core import TTmat
+    from TensorToolbox.core import TTvec, WTTvec, TTmat
     
     if ord == 'fro':
-        if isinstance(A,TTvec):            
+        if isinstance(A,TTvec) and not isinstance(A,WTTvec):
             # For the Frobenius norm in general we need to make the right to left rounding part
             # and store the Frobenius norms of the R matrices (????)
             d = len(A.TT)
@@ -138,7 +136,40 @@ def norm(A,ord='fro',round_eps=1e-10,eps=1e-4,maxit=1000,pow_guess=None,info=Fal
                 C = np.reshape(np.dot(C,R),(A.TT[k-1].shape[0],A.TT[k-1].shape[1],betakm1))
             nrm[0] = npla.norm(C.flatten(),2)
             return np.prod(nrm)
+
+        elif isinstance(A,WTTvec):
+            # Take the Frobenious norm of a weighted tensor
+            weights_applied = False
+            if not A.is_weighted():
+                A.apply_weights()
+                weights_applied = True
             
+            # For the Frobenius norm in general we need to make the right to left rounding part
+            # and store the Frobenius norms of the R matrices (????)
+            d = len(A.TT)
+            ns = A.shape()
+            # Right to left orthogonalization
+            nrm = np.empty(d,dtype=np.float64)
+            C = A.TT[d-1]
+            for k in range(d-1,0,-1):
+                # Computation of rq
+                alphakm1 = C.shape[0]
+                betak = C.shape[2]
+                Gk = np.reshape(C,(alphakm1,C.shape[1]*betak))
+                (R,Q) = scla.rq(Gk,mode='economic') # Only R is needed
+                betakm1 = R.shape[1]
+                nrm[k] = npla.norm(R,'fro')
+                R = R / max(nrm[k],1e-300)
+                # 3-mode product G[k-1] x_3 R
+                C = np.reshape(A.TT[k-1],(A.TT[k-1].shape[0]*A.TT[k-1].shape[1],A.TT[k-1].shape[2]))
+                C = np.reshape(np.dot(C,R),(A.TT[k-1].shape[0],A.TT[k-1].shape[1],betakm1))
+            nrm[0] = npla.norm(C.flatten(),2)
+
+            if weights_applied:
+                A.remove_weights()
+            
+            return np.prod(nrm)            
+
         elif isinstance(A,np.ndarray):
             return np.sqrt(np.sum(A**2.))
         else:
@@ -211,9 +242,7 @@ def sd(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
     ch.setFormatter(formatter)
     logger.addHandler(ch)
 
-    from TensorToolbox.core import TTvec
-    from TensorToolbox.core import zerosvec
-    from TensorToolbox.core import TTmat
+    from TensorToolbox.core import TTvec, TTmat, zerosvec
 
     if isinstance(A,TTmat):
         if not A.init : raise NameError("TensorToolbox.multilinalg.cg: TT not initialized correctly")
@@ -302,9 +331,7 @@ def cg(A,b,x0=None,eps=1e-8,maxit=1000,eps_round=1e-10,ext_info=False):
     ch.setFormatter(formatter)
     logger.addHandler(ch)
 
-    from TensorToolbox.core import TTvec
-    from TensorToolbox.core import zerosvec
-    from TensorToolbox.core import TTmat
+    from TensorToolbox.core import TTvec, TTmat, zerosvec
 
     if isinstance(A,TTmat):
         if not A.init : raise NameError("TensorToolbox.multilinalg.cg: TT not initialized correctly")
@@ -535,9 +562,7 @@ def gmres(A,b,x0=None,eps=1e-8,maxit=1000,restart=1000,eps_round=1e-10,ext_info=
     ch.setFormatter(formatter)
     logger.addHandler(ch)
 
-    from TensorToolbox.core import TTvec
-    from TensorToolbox.core import zerosvec
-    from TensorToolbox.core import TTmat
+    from TensorToolbox.core import TTvec, TTmat, zerosvec
 
     if not A.init or not b.init: raise NameError("TensorToolbox.multilinalg.cg: TT not initialized correctly")
     if not (isinstance(A,TTmat) and isinstance(b,TTvec) and not isinstance(b,TTmat)):
