@@ -15,7 +15,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -33,7 +33,7 @@ import time
 
 import TensorToolbox as DT
 import TensorToolbox.multilinalg as mla
-from aux import print_ok, print_fail, print_summary
+from TensorToolbox.unittests.auxiliary import print_ok, print_fail, print_summary
 
 def run(maxprocs, PLOTTING=False, loglev=logging.WARNING):
 
@@ -436,7 +436,7 @@ def run(maxprocs, PLOTTING=False, loglev=logging.WARNING):
                 else: tmp = sp.kron(tmp,D_sp)
             FULL_LAP = FULL_LAP + tmp
     except MemoryError:
-        print "FULL CG: Memory Error"
+        print("FULL CG: Memory Error")
         dofull = False
 
     # Construct Right hand-side (b=1, Dirichlet BC = 0)
