@@ -17,21 +17,25 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
 # Author: Daniele Bigoni
 #
 
-import run_unit_tests
-from run_unit_tests import *
-import TestTT
-from TestTT import *
+from TensorToolbox.unittests import run_unit_tests
+from TensorToolbox.unittests.run_unit_tests import *
+from TensorToolbox.unittests import TestTT
+from TensorToolbox.unittests.TestTT import *
+from TensorToolbox.unittests import TestWTT
+from TensorToolbox.unittests.TestWTT import *
+
 
 __all__ = []
 __all__ += run_unit_tests.__all__
 __all__ += TestTT.__all__
+__all__ += TestWTT.__all__
 
 __author__ = "Daniele Bigoni"
 __copyright__ = """Copyright 2014, The Technical University of Denmark"""
