@@ -15,7 +15,7 @@
 # along with TensorToolbox.  If not, see <http://www.gnu.org/licenses/>.
 #
 # DTU UQ Library
-# Copyright (C) 2014 The Technical University of Denmark
+# Copyright (C) 2014-2016 The Technical University of Denmark
 # Scientific Computing Section
 # Department of Applied Mathematics and Computer Science
 #
@@ -24,121 +24,147 @@
 
 import logging
 
-__all__ = ['RunUnitTests','RunTestTT','RunTestTensorWrapper','RunTestTTcross','RunTestTTdmrg','RunTestTTdmrgcross','RunTestQTTdmrg','RunTestQTT','RunTestSTTcross_0D', 'RunTestSTTcross_2D','RunTestSTTdmrg_0D', 'RunTestSTTdmrg_2D','RunTestSTTdmrgcross_0D','RunTestSQTTdmrg_0D']
+__all__ = ['RunUnitTests','RunTestTT','RunTestWTT','RunTestTensorWrapper','RunTestWeightedTensorWrapper','RunTestTTcross','RunTestTTdmrg','RunTestWTTdmrg','RunTestTTdmrgcross','RunTestQTTdmrg','RunTestQTT','RunTestSTTcross_0D', 'RunTestSTTcross_2D','RunTestSTTdmrg_0D', 'RunTestSTTdmrg_2D','RunTestSTTdmrgcross_0D','RunTestSQTTdmrg_0D']
 
 def RunTestTT(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestTT
     
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :returns: tuple (n success,n fail)
+    """
+    from TensorToolbox.unittests import TestTT
+    return TestTT.run(maxprocs, loglev=loglev)
+
+def RunTestWTT(maxprocs=None, loglev=logging.WARNING):
+    """ Runs the TestWTT
+    
     :params int maxprocs: If MPI support is enabled, defines how many processors to use.
     :returns: tuple (n success,n fail)
     """
-    import TestTT
-    return TestTT.run(maxprocs, loglev=loglev)
+    from TensorToolbox.unittests import TestWTT
+    return TestWTT.run(maxprocs, loglev=loglev)
 
 def RunTestTensorWrapper(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestTensorWrapper
     
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :returns: tuple (n success,n fail)
+    """
+    from TensorToolbox.unittests import TestTensorWrapper
+    return TestTensorWrapper.run(maxprocs,loglev=loglev)
+
+def RunTestWeightedTensorWrapper(maxprocs=None, loglev=logging.WARNING):
+    """ Runs the TestWeightedTensorWrapper
+    
     :params int maxprocs: If MPI support is enabled, defines how many processors to use.
     :returns: tuple (n success,n fail)
     """
-    import TestTensorWrapper
-    return TestTensorWrapper.run(maxprocs,loglev=loglev)
+    from TensorToolbox.unittests import TestWeightedTensorWrapper
+    return TestWeightedTensorWrapper.run(maxprocs,loglev=loglev)
 
 def RunTestTTcross(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestTTcross
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestTTcross
+    from TensorToolbox.unittests import TestTTcross
     return TestTTcross.run(maxprocs, loglev=loglev)
    
 def RunTestTTdmrg(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestTTdmrg
     
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
+    """
+    from TensorToolbox.unittests import TestTTdmrg
+    return TestTTdmrg.run(maxprocs, loglev=loglev)
+
+def RunTestWTTdmrg(maxprocs=None, loglev=logging.WARNING):
+    """ Runs the TestWTTdmrg
+    
     :params int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestTTdmrg
-    return TestTTdmrg.run(maxprocs, loglev=loglev)
+    from TensorToolbox.unittests import TestWTTdmrg
+    return TestWTTdmrg.run(maxprocs, loglev=loglev)
 
 def RunTestTTdmrgcross(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestTTdmrg
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestTTdmrgcross
+    from TensorToolbox.unittests import TestTTdmrgcross
     return TestTTdmrgcross.run(maxprocs, loglev=loglev)
 
 def RunTestQTTdmrg(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestQTTdmrg
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestQTTdmrg
+    from TensorToolbox.unittests import TestQTTdmrg
     return TestQTTdmrg.run(maxprocs, loglev=loglev)
 
 def RunTestQTT(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestQTT
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestQTT
+    from TensorToolbox.unittests import TestQTT
     return TestQTT.run(maxprocs, loglev=loglev)
 
 def RunTestSTTcross_0D(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestSTTcross_0D
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestSTTcross_0D
+    from TensorToolbox.unittests import TestSTTcross_0D
     return TestSTTcross_0D.run(maxprocs, loglev=loglev)
 
 def RunTestSTTcross_2D(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestSTTcross_2D
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestSTTcross_2D
+    from TensorToolbox.unittests import TestSTTcross_2D
     return TestSTTcross_2D.run(maxprocs, loglev=loglev)
 
 def RunTestSTTdmrg_0D(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestSTTdmrg_0D
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestSTTdmrg_0D
+    from TensorToolbox.unittests import TestSTTdmrg_0D
     return TestSTTdmrg_0D.run(maxprocs, loglev=loglev)
 
 def RunTestSTTdmrg_2D(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestSTTdmrg_2D
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestSTTdmrg_2D
+    from TensorToolbox.unittests import TestSTTdmrg_2D
     return TestSTTdmrg_2D.run(maxprocs, loglev=loglev)
 
 def RunTestSTTdmrgcross_0D(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestSTTdmrgcross_0D
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestSTTdmrgcross_0D
+    from TensorToolbox.unittests import TestSTTdmrgcross_0D
     return TestSTTdmrgcross_0D.run(maxprocs, loglev=loglev)
 
 def RunTestSQTTdmrg_0D(maxprocs=None, loglev=logging.WARNING):
     """ Runs the TestSTTdmrg_0D
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
-    import TestSQTTdmrg_0D
+    from TensorToolbox.unittests import TestSQTTdmrg_0D
     return TestSQTTdmrg_0D.run(maxprocs, loglev=loglev)
 
 def RunUnitTests(maxprocs=None,loglev=logging.WARNING):
     """ Runs all the unit tests.
     
-    :params int maxprocs: If MPI support is enabled, defines how many processors to use.
+    :param int maxprocs: If MPI support is enabled, defines how many processors to use.
     """
     
-    from aux import print_summary
+    from TensorToolbox.unittests.auxiliary import print_summary
 
     nsucc = 0
     nfail = 0
@@ -147,10 +173,18 @@ def RunUnitTests(maxprocs=None,loglev=logging.WARNING):
     nsucc += ns
     nfail += nf
 
+    (ns,nf) = RunTestWeightedTensorWrapper(maxprocs,loglev=loglev)
+    nsucc += ns
+    nfail += nf
+
     (ns,nf) = RunTestTT(maxprocs,loglev=loglev)
     nsucc += ns
     nfail += nf
 
+    (ns,nf) = RunTestWTT(maxprocs,loglev=loglev)
+    nsucc += ns
+    nfail += nf
+
     (ns,nf) = RunTestQTT(maxprocs,loglev=loglev)
     nsucc += ns
     nfail += nf
@@ -163,6 +197,10 @@ def RunUnitTests(maxprocs=None,loglev=logging.WARNING):
     nsucc += ns
     nfail += nf
 
+    (ns,nf) = RunTestWTTdmrg(maxprocs,loglev=loglev)
+    nsucc += ns
+    nfail += nf
+
     (ns,nf) = RunTestTTdmrgcross(maxprocs,loglev=loglev)
     nsucc += ns
     nfail += nf
